/*
    JW browser detection.
    
    Copyright (C) 2012 Egor Nepomnyaschih
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

JW.UI.Browsers = (function()
{
    ua = navigator.userAgent.toLowerCase();
    function check(r){
        return r.test(ua);
    };
    
    var isStrict = document.compatMode == "CSS1Compat",
        docMode = document.documentMode,
        isOpera = check(/opera/),
        isOpera10_5 = isOpera && check(/version\/10\.5/),
        isChrome = check(/\bchrome\b/),
        isWebKit = check(/webkit/),
        isSafari = !isChrome && check(/safari/),
        isSafari2 = isSafari && check(/applewebkit\/4/), // unique to Safari 2
        isSafari3 = isSafari && check(/version\/3/),
        isSafari4 = isSafari && check(/version\/4/),
        isSafari5 = isSafari && check(/version\/5/),
        isIE = !isOpera && check(/msie/),
        isIE7 = isIE && (check(/msie 7/) || docMode == 7),
        isIE8 = isIE && (check(/msie 8/) && docMode != 7 && docMode != 9 || docMode == 8),
        isIE9 = isIE && (check(/msie 9/) && docMode != 7 && docMode != 8 || docMode == 9),
        isIE6 = isIE && check(/msie 6/),
        isGecko = !isWebKit && check(/gecko/),
        isGecko2 = isGecko && check(/rv:1\.8/),
        isGecko3 = isGecko && check(/rv:1\.9/),
        isGecko4 = isGecko && check(/rv:2\.0/),
        isGecko5 = isGecko && check(/rv:5\./),
        isFF3_0 = isGecko3 && check(/rv:1\.9\.0/),
        isFF3_5 = isGecko3 && check(/rv:1\.9\.1/),
        isFF3_6 = isGecko3 && check(/rv:1\.9\.2/),
        isBorderBox = isIE && !isStrict,
        isWindows = check(/windows|win32/),
        isMac = check(/macintosh|mac os x/),
        isAir = check(/adobeair/),
        isLinux = check(/linux/),
        isIPad = check(/ipad/),
        isSecure = /^https/i.test(window.location.protocol);
    
    var isPaddingWideTd = !(isWebKit || isIPad);
    
    return {
        isStrict    : isStrict,
        isOpera     : isOpera,
        isOpera10_5 : isOpera10_5,
        isChrome    : isChrome,
        isWebKit    : isWebKit,
        isSafari    : isSafari,
        isSafari2   : isSafari2,
        isSafari3   : isSafari3,
        isSafari4   : isSafari4,
        isSafari5   : isSafari5,
        isIE        : isIE,
        isIE7       : isIE7,
        isIE8       : isIE8,
        isIE9       : isIE9,
        isIE6       : isIE6,
        isGecko     : isGecko,
        isGecko2    : isGecko2,
        isGecko3    : isGecko3,
        isGecko4    : isGecko4,
        isGecko5    : isGecko5,
        isFF3_0     : isFF3_0,
        isFF3_5     : isFF3_5,
        isFF3_6     : isFF3_6,
        isBorderBox : isBorderBox,
        isWindows   : isWindows,
        isMac       : isMac,
        isAir       : isAir,
        isLinux     : isLinux,
        isIPad      : isIPad,
        isSecure    : isSecure,
        
        isPaddingWideTd    : isPaddingWideTd
    };
})();
